#include <SoftwareSerial.h>
#include "protocol.h"

byte wordset = TWO;        // Wordset 2
byte timeout = FIVE;       // 5 second timeout

const int rxpin = 6;       // Receive pin (Rx)
const int txpin = 7;       // Transmit pin (Tx)
SoftwareSerial serial_vr(rxpin, txpin);
int loop_counter = 0;
char c;

void setup() {
  Serial.begin(9600);
  serial_vr.begin(9600);   // Serial to VR
  Serial.println("Setup complete");
  
  delay(100);
      
  c = vrQuery(CMD_BREAK);  // Wake up VR
  vrStatus("Awake: ", c, STS_SUCCESS);

  c = vrQuery(CMD_ID);     // Ask for ID
  vrStatus("Firmware identification: ", c, STS_ID);
    
  c = vrQuery(ARG_ACK);    // Send acknowledge
  vrStatus("Version: ", c, 0);

  serial_vr.write(CMD_LANGUAGE); 
  c = vrQuery(ZERO);       // ZERO: 0, English
  vrStatus("Set language: ", c, STS_SUCCESS);

  serial_vr.write(CMD_TIMEOUT);
  c = vrQuery(timeout);    // Timeout: 5 seconds
  vrStatus("Set timeout: ", c, STS_SUCCESS);
}

void loop() {
  
  Serial.println("");
  Serial.println("Command:");
  switch(wordset) {
    case ONE:
      Serial.println("action, move, turn, run");
      Serial.println("look, attack, stop, hello");
      break;
    case TWO:
      Serial.println("left, right, up, down");
      Serial.println("forward, backward");
      break;
    case THREE:
      Serial.println("0 through 10");
      break;
  }
  
  Serial.print("loop:");
  Serial.println(loop_counter++);

  serial_vr.write(CMD_RECOG_SI);   // Start recognition
  c = vrQuery(wordset);            // Set wordset
  
  switch(c) {
    case STS_TIMEOUT:
      Serial.println("(STS_TIMEOUT)");
      break;
    case STS_ERROR:
      Serial.print("(STS_ERROR) ");
      c = vrQuery(ARG_ACK);
      Serial.print(c);
      c = vrQuery(ARG_ACK);
      Serial.println(c);
      break;
   case STS_SIMILAR:
     Serial.println("Successful recognition (STS_SIMILAR)");
     c = vrQuery(ARG_ACK);
     switch(wordset) {
       case ONE:
         vr_1(c);
         break;
       case TWO:
         vr_2(c);
         break;
       case THREE:
         vr_3(c);
         break;       
     }
     break;
   default:
     Serial.print(c);
     Serial.println("Command Not Recognized");
     break;
  }  
  delay(500); 
  Serial.println("");
}

void vr_1(byte c) {
  Serial.print(c);
  switch (c) {
   case WORDSET_1_ACTION:
     Serial.println(" (WORDSET_1_ACTION)");
     break;
   case WORDSET_1_MOVE:
     Serial.println(" (WORDSET_1_MOVE)");
     break;
   case WORDSET_1_TURN:
     Serial.println(" (WORDSET_1_TURN)");
     break;
   case WORDSET_1_RUN:
     Serial.println(" (WORDSET_1_RUN)");
     break;
   case WORDSET_1_LOOK:
     Serial.println(" (WORDSET_1_LOOK)");
     break;
   case WORDSET_1_ATTACK:
     Serial.println(" (WORDSET_1_ATTACK)");
     break;
   case WORDSET_1_STOP:
     Serial.println(" (WORDSET_1_STOP)");
     break;
   case WORDSET_1_HELLO:
     Serial.println(" (WORDSET_1_HELLO)");
     break;
  }
}

void vr_2(byte c) {
  Serial.print(c);
  switch (c) {
   case WORDSET_2_LEFT:
     Serial.println(" (WORDSET_2_LEFT)");
     break;
   case WORDSET_2_RIGHT:
     Serial.println(" (WORDSET_2_RIGHT)");
     break;
   case WORDSET_2_UP:
     Serial.println(" (WORDSET_2_UP)");
     break;
   case WORDSET_2_DOWN:
     Serial.println(" (WORDSET_2_DOWN)");
     break;
   case WORDSET_2_FORWARD:
     Serial.println(" (WORDSET_2_FORWARD)");
     break;
   case WORDSET_2_BACKWARD:
     Serial.println(" (WORDSET_2_BACKWARD)");
     break;
  }
}

void vr_3(byte c) {
  Serial.print(c);
  switch (c) {
   case WORDSET_3_0:
     Serial.println(" (WORDSET_3_0)");
     break;
   case WORDSET_3_1:
     Serial.println(" (WORDSET_3_1)");
     break;
   case WORDSET_3_2:
     Serial.println(" (WORDSET_3_2)");
     break;
   case WORDSET_3_3:
     Serial.println(" (WORDSET_3_3");
     break;
   case WORDSET_3_4:
     Serial.println(" (WORDSET_3_4)");
     break;
   case WORDSET_3_5:
     Serial.println(" (WORDSET_3_5)");
     break;
   case WORDSET_3_6:
     Serial.println(" (WORDSET_3_6)");
     break;
   case WORDSET_3_7:
     Serial.println(" (WORDSET_3_7)");
     break;
   case WORDSET_3_8:
     Serial.println(" (WORDSET_3_8)");
     break;
   case WORDSET_3_9:
     Serial.println(" (WORDSET_3_9)");
     break;
   case WORDSET_3_10:
     Serial.println(" (WORDSET_3_10)");
     break;
  }
}

char vrQuery(byte cmd) {
  serial_vr.write(cmd);
  while (serial_vr.available() == 0) { } 
  return (serial_vr.read()); 
}

void vrStatus(String response, char c, byte expected) {
  Serial.print(response);
  Serial.print(c);
  Serial.print(" ");
  switch (expected) {
    case STS_SUCCESS:
      Serial.println("STS_SUCCESS");
      break;
    case STS_ID:
      Serial.println("STS_ID");
      break;
    default:
      if (c == VRBOT)
        Serial.println(("VRBOT")); 
      else if (c == EASYVR)
        Serial.println(("EASYVR"));        
  }
}
