#include <Servo.h> 
Servo servoLeft;          // Define left servo
Servo servoRight;         // Define right servo

#include <SoftwareSerial.h>
#include "protocol.h"

byte wordset = TWO;        // Wordset 2
byte timeout = TWO;        // 2 second timeout

const int ledActive = 5;   // LED control
const int rxpin = 6;       // Receive pin (Rx)
const int txpin = 7;       // Transmit pin (Tx)

SoftwareSerial serial_vr(rxpin, txpin);
char c;

void setup() {
  Serial.begin(9600);
  serial_vr.begin(9600);  // Serial to VR
  servoLeft.attach(10);   // Set left servo to digital pin 10
  servoRight.attach(11);  // Set right servo to digital pin 11
  pinMode(ledActive, OUTPUT);
  delay(100);
      
  c = vrQuery(CMD_BREAK);  // Wake up VR
  c = vrQuery(ARG_ACK);
  serial_vr.write(CMD_LANGUAGE); 
  c = vrQuery(ZERO);
  serial_vr.write(CMD_TIMEOUT);
  c = vrQuery(timeout); 
}

void loop() {
  
  digitalWrite(ledActive, HIGH);
  Serial.println("Command: left, right, up, down, forward, backward");
  serial_vr.write(CMD_RECOG_SI);   // Start recognition
  c = vrQuery(wordset);            // Set wordset
  digitalWrite(ledActive, LOW);
  
  switch(c) {
    case STS_TIMEOUT:
      Serial.println("(TIMEOUT)");
      ledBlink();
      break;
    case STS_ERROR:
      Serial.println("(ERROR) ");
      ledBlink();
      c = vrQuery(ARG_ACK);
      c = vrQuery(ARG_ACK);
      break;
   case STS_SIMILAR:
     Serial.print("Successful recognition: ");
     c = vrQuery(ARG_ACK);
     vr_2(c);
     break;
   default:
     ledBlink();
     Serial.println("Not recognized");
     break;
  }  
  delay(50); 
}

void vr_2(byte c) {
  Serial.print(c, DEC);
  switch (c) {
   case WORDSET_2_LEFT:
     Serial.println(" (LEFT)");
     botLeft();
     break;
   case WORDSET_2_RIGHT:
     Serial.println(" (RIGHT)");
     botRight();
     break;
   case WORDSET_2_UP:
     Serial.println(" (UP)");
     botForward();
     break;
   case WORDSET_2_DOWN:
     Serial.println(" (DOWN)");
     botStop();
     break;
   case WORDSET_2_FORWARD:
     Serial.println(" (FORWARD)");
     botForward();
     break;
   case WORDSET_2_BACKWARD:
     Serial.println(" (BACKWARD)");
     botReverse();
     break;
  }
}

char vrQuery(byte cmd) {
  serial_vr.write(cmd);
  while (serial_vr.available() == 0) { } 
  return (serial_vr.read()); 
}

void ledBlink() {
  digitalWrite(ledActive, HIGH);
  delay(125);
  digitalWrite(ledActive, LOW);
  delay(125);
  digitalWrite(ledActive, HIGH);
  delay(125);
  digitalWrite(ledActive, LOW);
  delay(125);
}


// Motion routines for forward, reverse, turns, and stop
void botForward() {
  servoLeft.write(180);
  servoRight.write(0); 
}
void botReverse() {
  servoLeft.write(0);
  servoRight.write(180);
}
void botRight() {
  servoLeft.write(180);
  servoRight.write(180);
}
void botLeft() {
  servoLeft.write(0);
  servoRight.write(0);
}
void botStop() {
  servoLeft.write(90);
  servoRight.write(90);
}
