#ifndef PROTOCOL_H
#define PROTOCOL_H

#define CMD_BAUDRATE    'a' // set baudrate <1> (bit time, 1=>115200)
#define CMD_BREAK       'b' // abort recog or ping
#define CMD_COUNT_SD    'c' // get command count for group <1>
#define CMD_RECOG_SD    'd' // do sd recog at group <1> (0 = trigger mixed si/sd)
#define CMD_ERASE_SD    'e' // reset command at group <1> pos <2>
#define CMD_GROUP_SD    'g' // insert new command at group <1> pos <2>
#define CMD_RECOG_SI    'i' // do si recog from ws <1>
#define CMD_KNOB        'k' // set si knob; 0-4
#define CMD_LANGUAGE    'l' // set si language <1>
#define CMD_MASK_SD     'm' // get active group mask
#define CMD_NAME_SD     'n' // label command at group <1> pos <2> with length <3> name <4-n>
#define CMD_TIMEOUT     'o' // set timeout <1>
#define CMD_DUMP_SD     'p' // read command data at group <1> pos <2>
#define CMD_RESETALL    'r' // reset all commands and groups
#define CMD_SLEEP       's' // go to power down
#define CMD_TRAIN_SD    't' // train sd command at group <1> pos <2>
#define CMD_UNGROUP_SD  'u' // remove command at group <1> pos <2>
#define CMD_LEVEL       'v' // set sd level; 1-5
#define CMD_ID          'x' // get version id
#define CMD_DELAY       'y' // set transmit delay <1> (log scale)

#define STS_COUNT       'c' // count of commands <1>
#define STS_DATA        'd' // provide command length <1> label <2-11> training <12>
#define STS_ERROR       'e' // signal error code <1-2>
#define STS_INTERR      'i' // back from aborted recognition (see 'break')
#define STS_MASK        'k' // mask of active groups <1-8>
#define STS_OUT_OF_MEM  'm' // no more available commands (see 'group')
#define STS_SUCCESS     'o' // no errors status
#define STS_RESULT      'r' // recognised sd command <1> - training similar to sd <1>
#define STS_SIMILAR     's' // recognised si <1> (in mixed si/sd) - training similar to si <1>
#define STS_TIMEOUT     't' // timeout expired
#define STS_INVALID     'v' // invalid command or argument
#define STS_AWAKEN      'w' // back from power down mode
#define STS_ID          'x' // provide version id <1>

// protocol arguments are in the range 0x40 (-1) to 0x60 (+31) inclusive
#define ARG_MIN     0x40
#define ARG_MAX     0x60
#define ARG_ZERO    0x41
#define ARG_ACK     0x20    // to read more status arguments

#define MINUS_ONE         '@' // 0x40
#define ZERO              'A' // 0x41
#define ONE               'B' // 0x42
#define TWO               'C' // 0x43
#define THREE             'D' // 0x44
#define FOUR              'E' // 0x45
#define FIVE              'F' // 0x46
#define SIX               'G' // 0x47
#define SEVEN             'H' // 0x48

#define WORDSET_1_ACTION  'A' // 0x41
#define WORDSET_1_MOVE    'B' // 0x42
#define WORDSET_1_TURN    'C' // 0x43
#define WORDSET_1_RUN     'D' // 0x44
#define WORDSET_1_LOOK    'E' // 0x45
#define WORDSET_1_ATTACK  'F' // 0x46
#define WORDSET_1_STOP    'G' // 0x47
#define WORDSET_1_HELLO   'H' // 0x48

#define WORDSET_2_LEFT    'A' // 0x41
#define WORDSET_2_RIGHT   'B' // 0x42
#define WORDSET_2_UP      'C' // 0x43
#define WORDSET_2_DOWN     'D' // 0x44
#define WORDSET_2_FORWARD  'E' // 0x45 
#define WORDSET_2_BACKWARD 'F' // 0x46

#define WORDSET_3_0        'A' // 0x41
#define WORDSET_3_1        'B' // 0x42
#define WORDSET_3_2        'C' // 0x43
#define WORDSET_3_3        'D' // 0x44
#define WORDSET_3_4        'E' // 0x45
#define WORDSET_3_5        'F' // 0x46
#define WORDSET_3_6        'G' // 0x47
#define WORDSET_3_7        'H' // 0x48
#define WORDSET_3_8        'I' // 0x49
#define WORDSET_3_9        'J' // 0x4A
#define WORDSET_3_10       'K' // 0x4B

#define VRBOT              'A' // 0x41
#define EASYVR             'B' // 0x42

#endif //PROTOCOL_H

